#!/bin/bash

debug() {
    #
    # You already know what it does.
    #
    # BTW, following are equivalent:
    #
    #     debug "var1=$var1" "var2=$var2" "result=$result"
    #     debug -v var1:var2:result
    #     debug -v "var@:result"
    #
    $FFOOD_DEBUG || return 0
    if test "$1" == "-v";
    then
        local varnames="$2"
        local vn
        for vn in $(tr ":" " " <<<"$varnames");
        do
            local heel="${vn:0-1}"          # last char
            local toes="${vn%%$heel}"       # all but last char
            if test "$heel" == "@";
            then
                # FIXME: review+fix the eval (even at cost of dropping the feature)
                local vars=$(eval "echo \"\${!$toes$heel}\"")
                debug -v $(tr " " ":" <<<"$vars")
            else
                debug "$vn='$(echo -n ${!vn})'"
            fi
        done
    elif test "$1" == "-t";
    then
        local -a tr=(${FUNCNAME[@]})
        unset tr[0]; unset tr[1];
        local tmp=$(echo "${tr[*]}" | tr '  ' '\n' | tac )
        echo "== trace ==" >&2
        echo "$tmp" | sed -e 's/^/-> /g' >&2
    else
        local me=${FUNCNAME[1]}
        case $me in
            main)       me=$(basename $0)       ;;
            debug)      me="${FUNCNAME[2]}()"   ;;
            debug_pipe) me="${FUNCNAME[2]}()"   ;;
            *)          me="$me()"              ;;
        esac
        local l
        for l in "$@";
        do echo "debug:$me: $l" >&2
        done
    fi
}

debug_pipe() {
    #
    # Debug the whole pipe.
    #
    grep ".*" \
        | while IFS= read line;
          do
              debug "PIPE $1: '$line'"
              echo "$line"
          done
}

die() {
    #
    # A Perl-style death (a truly honorable one that is)
    #
    debug -t
    warn $*
    exit 9
}

echo_hr() {
    #
    # Echo a horizontal ruler out of char "$1" all across terminal.
    #
    test 0$COLUMNS -gt 0 || return
    local char="$1"
    local i
    for (( i=1; $i<$COLUMNS; i=$i+1 )); do echo -n "$char"; done
    echo
}

mute_known() {
    #
    # Mute known messages
    #
    # For those yums and rpms that don't know theit manners.  Use with care!
    #
    grep -vxf <(iniread -s mute -k $1 mute.ini)
}

usage_is() {
    #
    # Echo out usage patterns and exit 1
    #
    # Define usage() in your script and use this in body
    #
    local me=$(basename $0)
    local u
    for u in "$@";
    do echo "usage: $me $u"
    done
    exit 1
}

usagef() {
    #
    # Same as usage_is, but for functions.
    #
    # That means it does not exit and uses FUNCNAME, not $0
    #
    local u
    for u in "$@";
    do echo "usage: ${FUNCNAME[1]} $u"
    done
}

think() {
    #
    # If verose is on, think loud.
    #
    $FFOOD_VERBOSE && echo "$@"
}

warn() {
    #
    # Warn them
    #
    # Resist  temptation to prefix "warning" or similar BS here.
    # STDERR is *made* for warnings.
    #
    echo "$*" >&2
}

append_if_missing() {
    #
    # Append line to the file but only if it's not already there.
    #
    # Handy for your fstabs, or whatever line-based confs Handy for your fstabs, or whatever line-based confs
    #
    local line=$1
    local file=$2
    grep -qsx "$line" $file || echo "$line" >> $file
}

collapse_tilde() {
    #
    # Exchange home back to "~".
    #
    # Nice to save them users from some eye pain.
    #
    perl -pe "s|^$HOME|~|"
}

expand_tilde() {
    #
    # Exchange "~" for home.
    #
    perl -pe "s|^[[:space:]]*~|$HOME|"
}

__iniread__cat() {
    # cat without starting a process
    while IFS= read line;
    do echo "$line"; done
}

__iniread__flt_comments() {
    grep -v -e "^[[:space:]]*[#;]" -e "^[[:space:]]*$"
}

__iniread__flt_key() {
    # filter values from key=value pair
    local line
    local key
    local value
    while IFS= read line;
    do
        line=$(sed -re 's/^\s*//' <<<"$line")
        key=$(sed -re 's/\s*=.*//; s/^\s*//' <<<"$line")
        if $strict;
        then
            value=$(sed -re 's/[^=]*=//' <<<"$line")
        else
            value=$(sed -re 's/[^=]*=\s*//' <<<"$line")
        fi
        if test "$key" = "$wantkey";
        then
            echo "$value"
        fi
    done
}

__iniread__flt_section() {
    # filter per section
    local section_ok=false
    local line
    while IFS= read line;
    do
        if grep -qse "^\[$wantsection\]" <<<"$line";
        then
            section_ok=true
        elif grep -qse "^\[" <<<"$line";
        then
            section_ok=false
        elif $section_ok;
        then
            $strict || line="$(sed -re 's/^\s*//' <<<"$line")"
            echo "$line"
        fi
    done
}

__iniread__merge() {
    local spec="$1"
    local file=""
    local dirs="/etc/ffood $HOME/.ffood"
    local p
    case $spec in
        -|*/*)          # stdin, or path (with slash)
            cat "$spec"
            ;;
        "")             # name not given
            for p in $dirs;
            do
                debug -v p
                cat $p/ffood.ini 2>/dev/null
            done
            cat .ffood.ini 2>/dev/null
            ;;
        *)              # name given
            for p in $dirs;
            do
                debug -v p
                cat $p/$spec 2>/dev/null
            done
            ;;
    esac
    true
}

iniread() {
    #
    # A flexible (not just) .ini file reader
    #
    # Can read anything from commented plaintext to keyless section INI,
    # sectionless keyed INI or sectioned keyed INI, including multi-line
    # and whitespace-safe chunks.
    #
    # Is constantly becoming smarter and smarter.
    #
    local file="-"
    local wantsection=""
    local wantkey=""
    local strict=false
    local section_ok=true
    local one_line=false
    while true;
    do
        case $1 in
            -1|--one-line)
                one_line=true
                shift 1
                ;;
            -k|--key)
                wantkey="$2"
                shift 2
                ;;
            -p|--path)
                wantkey="$(echo $2 | rev | cut -d. -f1 | rev)"
                wantsection="${2%.$wantkey}"
                shift 2
                ;;
            -s|--section)
                wantsection="$2"
                section_ok=false
                shift 2
                ;;
            -S|--strict)
                strict=true
                shift 1
                ;;
            -l|--list-sections)
                #TODO: list sections
                warn "--list-sections is not implemented"
                shift
                ;;
            -L|--list-keys)
                #TODO: list keys as section/key (with -s, list keys only)
                warn "--list-keys is not implemented"
                shift
                ;;
            --)
                file=$2
                break
                ;;
            "")
                break
                ;;
            *)
                file=$1
                break
                ;;
        esac
    done

    flt_section=__iniread__cat
    flt_key=__iniread__cat
    limit_line=__iniread__cat
    test -n "$wantsection" && flt_section=__iniread__flt_section
    test -n "$wantkey" && flt_key=__iniread__flt_key
    $one_line && limit_line="head -1"

    debug -v file:wantkey:wantsection

    __iniread__merge $file \
        | __iniread__flt_comments \
        | $flt_section \
        | $flt_key \
        | $limit_line
}
