#!/bin/bash

ffood import core


age_of_leaf() {
    local now=$1
    sed 's/.*/"&"/' \
        | xargs stat -c %Z \
        | while read ctime;
          do echo $(($now - $ctime));
          done
}

age_of_tree() {
    local now=$(date +%s)
    find -L "$1" \
        | age_of_leaf $now \
        | sort -n \
        | head -1
}

cat_uri() {
    wget -q $1 -O - || echo "ERROR: $?" >&2
}

filter() {
    local silent
    while true; do case $1 in
        -s|--silent) silent=true; shift ;;
        *) break ;;
    esac done
    debug "\$*=$*"
    while read item;
    do
        if $* $item;
        then
            echo "$item"
        fi
    done
}

pids_matching() {
    local all_pids=""
    local pids=""
    local pfx=""
    local pattern
    for pattern in "$@";
    do
        local pids=$(ps -e -o pid= -o comm= \
                     | grep $pattern \
                     | sed -e 's/^ *//' \
                     | cut -d\  -f 1 \
                     | tr "\n" " ")
        debug -v pattern:pids
        all_pids="$all_pids$pfx$pids"
        pfx=" "
    done
    debug -v all_pids
    echo $all_pids
    test -n "$all_pids"
}

port_listening() {
    local host="localhost"
    local port="$1"
    debug -v host:port
    test -n "$port" || usagef "PORT"
    netstat -ntlp \
        | sed -e 's/  */ /g' \
        | cut -d\  -f 4 \
        | grep -qs :$port\$
}

reachable_by_ping() {
    local host=$1
    debug -v host
    test -n "$host" || usagef "HOST"
    ping -w 1 $host >& /dev/null
}

reachable_by_ssh() {
    local host=$1
    debug -v host
    test -n "$host" || usagef "HOST"
    ssh -n -o ConnectTimeout=3 $host -- true >/dev/null 2>&1;
}

reachable_by_tcp() {
    local port=$1
    local host=$2
    debug -v port:host
    test -n "$host" -a -n "$port" || usagef "PORT HOST"
    nc -w 1 $host $port >/dev/null 2>&1;
}

filter_hosts() {
    local do_ping=true
    local do_ssh=false
    local do_tcp=false
    local tcp_port=0

    while true;
    do
        case $1 in
            -t|--tcp)
                do_tcp=true
                tcp_port=$2
                shift 2
                ;;
            -p|--ping)
                do_ping=true
                shift 1
                ;;
            -P|--no-ping)
                do_ping=false
                shift 1
                ;;
            -s|--ssh)
                do_ssh=true
                shift 1
                ;;
            -S|--no-ssh)
                do_ssh=false
                shift 1
                ;;
            "")
                break
                ;;
            *)
                usagef "[-p|P] [-s|-S] [-t port]"
                return 2
                ;;
        esac
    done

    local ping_fun=true
    local ssh_fun=true
    local tcp_fun=true
    $do_ping    && ping_fun="reachable_by_ping"
    $do_tcp     && tcp_fun="reachable_by_tcp $tcp_port"
    $do_ssh     && ssh_fun="reachable_by_ssh"
    filter $ping_fun | filter $tcp_fun | filter $ssh_fun
}

sort_paths_by_age() {
    while read path;
    do echo "$(age_of_tree $path) $path"
    done \
      | sort -n \
      | cut -d\  -f 2-
}

wait_until() {
    local timeout=10
    local delay=1
    local use_eval=false
    while true; do case "$1" in
        -e) use_eval=true; shift; break ;;
        -d) delay="$2"; shift 2;        ;;
        -t) timeout="$2"; shift 2;      ;;
        --) shift 1; break;             ;;
        *)  break;                      ;;
    esac done
    think "waiting for: $@"
    local elapsed=0
    local start=$(date +%s)
    while true;
    do
        sleep $delay
        local now=$(date +%s)
        local elapsed=$(($now - $start))
        local rv=0
        if $use_eval;
        then
            eval $@;
            rv=$?
        else
            $@;
            rv=$?
        fi
        if [ $rv -eq 0 ];
        then
            return 0;
        fi
        if test $elapsed -gt $timeout;
        then
            warn "timeout reached when waiting: $elapsed > $timeout"
            return 1
        fi
        think "waiting for another $delay s"
    done
}
