#!/bin/bash

ffood import core

filter_ids() {
    #
    # Hack doer-like id's (ID#123) out of the text.
    #
    tr ' ' '\n' \
      | perl -ne '
            next unless m/\b([a-zA-Z]{2,}#\d+)\b/;
            print "$1\n";
        ' \
      | debug_pipe id_lookalikes
}

filter_kws() {
    #
    # Hack out lines that look like kw expressions.
    #
    local kwbase="[a-z][a-z0-9]*"
    grep -e "^$kwbase " -e "^$kwbase\$" \
        | debug_pipe kw_lookalikes
}

filter_uris() {
    #
    # Hack URIs out of the text.
    #
    tr ' ' '\n' \
      | perl -ne '
            next unless m|(\bhttps?://[[:alnum:]_:/.?&%=-]+)\b|;
            print "$1\n";
        ' \
      | debug_pipe uri_lookalikes
}

find_uri() {
    #
    # Scan stdin for what looks like URI, ID or keyword
    #
    # Output URIS: first what were apparent uris, then IDs converted to
    # URIs, then things that looked like kw expressions, converted to
    # URIs.
    #
    # Apply this filter to args or clipboard, and either use head -1 or
    # if you are brave, open all URIs.
    #
    local d=$(mktemp -d)
    pushd $d >&/dev/null
        ##
        # heat up and fill pipes
        #
        mkfifo maybe_uris maybe_ids maybe_kws \
               uris uris_from_ids uris_from_kws
        tee maybe_uris | tee maybe_ids | tee maybe_kws | cat >/dev/null &
        ##
        # process each pipe *async* by different filter
        #
        cat maybe_uris | filter_uris                       > uris &
        cat maybe_ids  | filter_ids  | tr '#' ' ' | kw2uri > uris_from_ids &
        cat maybe_kws  | filter_kws  | kw2uri              > uris_from_kws &
        ##
        # print result *sync* in correct order
        #
        cat uris
        cat uris_from_ids
        cat uris_from_kws
    popd >&/dev/null
    rm -rf $d
}

kw2uri() {
    #
    # Turn keyword or query (like "g hello" for google) to URI
    #
    local line
    declare -l line
    while read line;
    do
        line="$line "
        local kw=${line%% *}
        local query=${line#$kw }
        debug -v kw:query
        if test -n "$query";
        then
            local fmt=$(iniread -1 -p www.query.$kw www.ini)
            debug -v fmt
            printf "$fmt\n" "$query"
        else
            iniread -1 -p www.bookmark.$kw www.ini
        fi
    done
}

